﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.Hive.JobBase;
using HeuristicLab.Persistence.Default.Xml;
using System.IO;

namespace HeuristicLab.Hive.Experiment.Views {
  [View("JobItem View")]
  [Content(typeof(JobItem), true)]
  public partial class JobItemView : ItemView {
    public new JobItem Content {
      get { return (JobItem)base.Content; }
      set {
        if (base.Content != value) {
          base.Content = value;
        }
      }
    }

    public JobItemView() {
      InitializeComponent();
      this.ReadOnly = true;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content != null) {
        this.stateTextBox.Text = Content.JobDto.State.ToString();
        this.userIdTextBox.Text = Content.JobDto.UserId.ToString();
        this.percentageTextBox.Text = Content.JobDto.Percentage.ToString();
        this.dateCreatedTextBox.Text = Content.JobDto.DateCreated.ToString();
        this.dateCalculatedText.Text = Content.JobDto.DateCalculated.ToString();
        this.dateFinishedTextBox.Text = Content.JobDto.DateFinished.ToString();
        this.priorityTextBox.Text = Content.JobDto.Priority.ToString();
        this.coresNeededTextBox.Text = Content.JobDto.CoresNeeded.ToString();
        this.memoryNeededTextBox.Text = Content.JobDto.MemoryNeeded.ToString();
        this.pluginsNeededTextBox.Text = string.Join(", ", Content.JobDto.PluginsNeeded.Select(x => x.Name + "-" + x.Version).ToArray());
        this.projectTextBox.Text = Content.JobDto.Project != null ? Content.JobDto.Project.Name : "-";
        if (Content.LatestSnapshot != null) {
          snapshotStatusText.Text = Content.LatestSnapshot.StatusMessage;
          snapshotTimeText.Text = Content.LatestSnapshotTime.ToString();
        }
      }
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.LatestSnapshotChanged += new EventHandler(Content_LatestSnapshotChanged);
    }

    protected override void DeregisterContentEvents() {
      Content.LatestSnapshotChanged -= new EventHandler(Content_LatestSnapshotChanged);
      base.DeregisterContentEvents();
    }

    void Content_LatestSnapshotChanged(object sender, EventArgs e) {
      if (Content.LatestSnapshot != null) {
        snapshotStatusText.Text = Content.LatestSnapshot.StatusMessage;
        snapshotTimeText.Text = Content.LatestSnapshotTime.ToString();
      } else {
        snapshotStatusText.Text = "";
        snapshotTimeText.Text = "";
      }
      SetEnabledStateOfControls();
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      openSnapshotButton.Enabled = Content != null && Content.LatestSnapshot != null;
    }

    private void openSnapshotButton_Click(object sender, EventArgs e) {
      OptimizerJob job = XmlParser.Deserialize<OptimizerJob>(new MemoryStream(Content.LatestSnapshot.Obj.SerializedJobData));
      MainFormManager.MainForm.ShowContent(job.Optimizer);
    }
  }
}
